#include "tracking.h"

uint8_t TrackingController::baseSpeed = 10; // 默认速度50%
uint8_t TrackingController::turnSpeed = 10; // 默认转弯速度30%

void TrackingController::init()
{
  Serial.println("Tracking Init");
}

void TrackingController::setSpeed(uint8_t base, uint8_t turn)
{
  baseSpeed = base;
  turnSpeed = turn;
}

void TrackingController::update()
{
  IR::update();

  const IRData &irData = IR::data;
  float offset = calculateOffset(irData);

  Serial.print("Offset: ");
  Serial.println(offset);

  // 检查是否丢线
  if (offset == LOST_LINE)
  { // 需要定义 LOST_LINE 常量
    // 丢线时停止移动
    Serial.println("Lost line!");
    MotorController::motorXYRControl(0, 0, 0);
    return;
  }

  // 正常巡线控制
  float correction = PIDController::update(0, offset);
  MotorController::motorXYRControl(0, baseSpeed, -correction);
}

// 计算偏移量，返回范围 [-2, 2]
float TrackingController::calculateOffset(const IRData &irData)
{
  float offset = 0;
  int activeCount = 0;
  bool allHigh = true;
  bool allLow = true;

  Serial.print("IR Data: ");
  for (int i = 0; i < irData.length; i++) {
    Serial.print(irData.data[i]);
    Serial.print(" ");
  }
  Serial.println();

  // 检查是否所有传感器都是高电平或低电平
  for (int i = 0; i < irData.length; i++) {
    if (irData.data[i]) {
      allLow = false;
    } else {
      allHigh = false;
    }
  }

  // 如果所有传感器都是高电平或低电平，说明不在线上
  if (allHigh || allLow) {
    return LOST_LINE;
  }

  // 计算加权平均偏移
  for (int i = 0; i < irData.length; i++)
  {
    if (irData.data[i])
    {
      // 将传感器位置映射到 [-2, 2] 范围
      offset += (i - (irData.length - 1) / 2.0f) * (4.0f / (irData.length - 1));
      activeCount++;
    }
  }

  // 如果有传感器检测到线，返回平均偏移
  if (activeCount > 0)
    return offset / activeCount;

  return LOST_LINE;
}